@@ -32,22 +32,34 @@
 #include <libxml/parser.h>
 #include <libxml/xmlstring.h>
 
+#include <unique/unique.h>
+
+#include "brasero-misc.h"
+
 #include "brasero-app.h"
-#include "brasero-utils.h"
-#include "brasero-jacket-edit.h"
 #include "brasero-blank-dialog.h"
 #include "brasero-sum-dialog.h"
 #include "brasero-eject-dialog.h"
-#include "brasero-session.h"
 #include "brasero-project-manager.h"
+#include "brasero-xsession.h"
 #include "brasero-pref.h"
-#include "brasero-image-option-dialog.h"
-#include "brasero-disc-copy-dialog.h"
-#include "brasero-burn-dialog.h"
-#include "burn-debug.h"
+
 #include "brasero-drive.h"
-#include "burn-caps.h"
-#include "burn.h"
+
+#include "burn-debug.h"
+
+#include "brasero-tags.h"
+#include "brasero-burn.h"
+#include "brasero-track-disc.h"
+#include "brasero-track-image.h"
+#include "brasero-track-image-cfg.h"
+#include "brasero-session.h"
+#include "brasero-burn-lib.h"
+
+#include "brasero-burn-options.h"
+#include "brasero-burn-dialog.h"
+#include "brasero-jacket-edit.h"
+
 #include "burn-plugin-manager.h"
 
 typedef struct _BraseroAppPrivate BraseroAppPrivate;
@@ -117,14 +129,14 @@ static GtkActionEntry entries[] = {
 	{"Eject", "media-eject", N_("E_ject"), NULL,
 	 N_("Eject a disc"), G_CALLBACK (on_eject_cb)},
 
-	{"Erase", "media-optical-blank", N_("_Erase..."), NULL,
-	 N_("Erase a disc"), G_CALLBACK (on_erase_cb)},
+	{"Blank", "media-optical-blank", N_("_Blank..."), NULL,
+	 N_("Blank a disc"), G_CALLBACK (on_erase_cb)},
 
 	{"Check", NULL, N_("_Check Integrity..."), NULL,
 	 N_("Check data integrity of disc"), G_CALLBACK (on_integrity_check_cb)},
 
-	{"Exit", GTK_STOCK_QUIT, NULL, NULL,
-	 N_("Exit the program"), G_CALLBACK (on_exit_cb)},
+	{"Quit", GTK_STOCK_QUIT, NULL, NULL,
+	 N_("Quit Brasero"), G_CALLBACK (on_exit_cb)},
 
 	{"Contents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Display help"),
 	 G_CALLBACK (on_help_cb)}, 
@@ -140,7 +152,7 @@ static const gchar *description = {
 	    "<menu action='ProjectMenu'>"
 		"<placeholder name='ProjectPlaceholder'/>"
 		"<separator/>"
-		"<menuitem action='Exit'/>"
+		"<menuitem action='Quit'/>"
 	    "</menu>"
 	    "<menu action='EditMenu'>"
 		"<placeholder name='EditPlaceholder'/>"
@@ -153,7 +165,7 @@ static const gchar *description = {
 	    "<menu action='ToolMenu'>"
 		"<placeholder name='DiscPlaceholder'/>"
 		"<menuitem action='Eject'/>"
-		"<menuitem action='Erase'/>"
+		"<menuitem action='Blank'/>"
 		"<menuitem action='Check'/>"
 	    "</menu>"
 	    "<menu action='HelpMenu'>"
@@ -406,7 +418,7 @@ brasero_app_save_contents (BraseroApp *app,
 
 	if (priv->burn_dialog) {
 		if (cancellable)
-			return (brasero_burn_dialog_cancel (BRASERO_BURN_DIALOG (priv->burn_dialog)) == FALSE);
+			return (brasero_burn_dialog_cancel (BRASERO_BURN_DIALOG (priv->burn_dialog), FALSE) == FALSE);
 
 		gtk_widget_destroy (priv->burn_dialog);
 		return FALSE;
@@ -648,7 +660,7 @@ brasero_app_is_running (BraseroApp *app)
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
-	return (priv->mainwin != NULL);
+	return priv->mainwin_running;
 }
 
 void
@@ -686,31 +698,41 @@ brasero_app_burn (BraseroApp *app,
 	return success;
 }
 
+gboolean
+brasero_app_burn_options (BraseroApp *app,
+			  BraseroSessionCfg *session)
+{
+	GtkWidget *dialog;
+	GtkResponseType result;
+
+	dialog = brasero_burn_options_new (session);
+	brasero_app_set_toplevel (app, GTK_WINDOW (dialog));
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	/* The destruction of the dialog will bring the main window forward */
+	gtk_widget_destroy (dialog);
+	return (result == GTK_RESPONSE_OK);
+}
+
 void
 brasero_app_burn_image (BraseroApp *app,
 			const gchar *uri)
 {
-	BraseroBurnSession *session;
-	GtkResponseType result;
-	GtkWidget *dialog;
+	BraseroSessionCfg *session;
+	BraseroTrackImageCfg *track;
 
 	/* setup, show, and run options dialog */
-	dialog = brasero_image_option_dialog_new ();
-	brasero_image_option_dialog_set_image_uri (BRASERO_IMAGE_OPTION_DIALOG (dialog), uri);
+	session = brasero_session_cfg_new ();
 
-	brasero_app_set_toplevel (app, GTK_WINDOW (dialog));
+	track = brasero_track_image_cfg_new ();
+	if (uri)
+		brasero_track_image_cfg_set_source (track, uri);
 
-	result = gtk_dialog_run (GTK_DIALOG (dialog));
-	if (result != GTK_RESPONSE_OK) {
-		gtk_widget_destroy (dialog);
-		return;
-	}
+	brasero_burn_session_add_track (BRASERO_BURN_SESSION (session), BRASERO_TRACK (track));
 
-	session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
-	/* The destruction of the dialog will bring the main window forward */
-	gtk_widget_destroy (dialog);
+	if (brasero_app_burn_options (app, session))
+		brasero_app_burn (app, BRASERO_BURN_SESSION (session));
 
-	brasero_app_burn (app, session);
 	g_object_unref (session);
 }
 
@@ -719,12 +741,12 @@ brasero_app_copy_disc (BraseroApp *app,
 		       const gchar *device,
 		       const gchar *cover)
 {
-	BraseroBurnSession *session;
-	GtkResponseType result;
-	GtkWidget *dialog;
+	BraseroSessionCfg *session;
+	BraseroTrackDisc *track;
 
-	dialog = brasero_disc_copy_dialog_new ();
-	brasero_app_set_toplevel (app, GTK_WINDOW (dialog));
+	session = brasero_session_cfg_new ();
+	track = brasero_track_disc_new ();
+	brasero_burn_session_add_track (BRASERO_BURN_SESSION (session), BRASERO_TRACK (track));
 
 	/* if a device is specified then get the corresponding medium */
 	if (device) {
@@ -735,21 +757,10 @@ brasero_app_copy_disc (BraseroApp *app,
 		drive = brasero_medium_monitor_get_drive (monitor, device);
 		g_object_unref (monitor);
 
-		brasero_disc_copy_dialog_set_drive (BRASERO_DISC_COPY_DIALOG (dialog), drive);
+		brasero_track_disc_set_drive (BRASERO_TRACK_DISC (track), drive);
 		g_object_unref (drive);
 	}
 
-	result = gtk_dialog_run (GTK_DIALOG (dialog));
-	if (result != GTK_RESPONSE_OK) {
-		gtk_widget_destroy (dialog);
-		return;
-	}
-
-	session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
-
-	/* The destruction of the dialog will bring the main window forward */
-	gtk_widget_destroy (dialog);
-
 	/* Set a cover if any. */
 	if (cover) {
 		GValue *value;
@@ -757,12 +768,14 @@ brasero_app_copy_disc (BraseroApp *app,
 		value = g_new0 (GValue, 1);
 		g_value_init (value, G_TYPE_STRING);
 		g_value_set_string (value, cover);
-		brasero_burn_session_tag_add (session,
+		brasero_burn_session_tag_add (BRASERO_BURN_SESSION (session),
 					      BRASERO_COVER_URI,
 					      value);
 	}
 
-	brasero_app_burn (app, session);
+	if (brasero_app_burn_options (app, session))
+		brasero_app_burn (app, BRASERO_BURN_SESSION (session));
+
 	g_object_unref (session);
 }
 
@@ -770,7 +783,7 @@ void
 brasero_app_blank (BraseroApp *app,
 		   const gchar *device)
 {
-	GtkWidget *dialog;
+	BraseroBlankDialog *dialog;
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
@@ -791,9 +804,9 @@ brasero_app_blank (BraseroApp *app,
 		g_object_unref (drive);
 	}
 
-	priv->tool_dialog = dialog;
+	priv->tool_dialog = GTK_WIDGET (dialog);
 	if (!priv->mainwin) {
-		gtk_widget_realize (dialog);
+		gtk_widget_realize (GTK_WIDGET (dialog));
 
 		if (priv->parent) {
 			gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
@@ -812,7 +825,7 @@ brasero_app_blank (BraseroApp *app,
 	}
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 	priv->tool_dialog = NULL;
 }
 
@@ -851,13 +864,13 @@ void
 brasero_app_check (BraseroApp *app,
 		   const gchar *device)
 {
-	GtkWidget *dialog;
+	BraseroSumDialog *dialog;
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
 
 	dialog = brasero_sum_dialog_new ();
-	priv->tool_dialog = dialog;
+	priv->tool_dialog = GTK_WIDGET (dialog);
 
 	if (device) {
 		BraseroDrive *drive;
@@ -875,7 +888,7 @@ brasero_app_check (BraseroApp *app,
 	}
 
 	if (!priv->mainwin) {
-		gtk_widget_realize (dialog);
+		gtk_widget_realize (GTK_WIDGET (dialog));
 
 		if (priv->parent) {
 			gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
@@ -894,7 +907,7 @@ brasero_app_check (BraseroApp *app,
 	}
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 	priv->tool_dialog = NULL;
 }
 
@@ -1149,6 +1162,7 @@ brasero_app_recent_open (GtkRecentChooser *chooser,
 		return;
 	}
 
+	/* Make sure it is no longer one shot */
 	brasero_project_manager_open_by_mime (BRASERO_PROJECT_MANAGER (priv->projects),
 					      uri,
 					      mime);
@@ -1214,7 +1228,7 @@ brasero_menu_item_selected_cb (GtkMenuItem *proxy,
 
 	priv = BRASERO_APP_PRIVATE (app);
 
-	action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
 	g_return_if_fail (action != NULL);
 
 	g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
@@ -1427,16 +1441,35 @@ brasero_app_create_mainwin (BraseroApp *app)
 	brasero_app_load_window_state (app);
 }
 
-void
+gboolean
 brasero_app_run_mainwin (BraseroApp *app)
 {
 	BraseroAppPrivate *priv;
+	UniqueApp *uapp;
 
 	priv = BRASERO_APP_PRIVATE (app);
 
 	priv->mainwin_running = 1;
 	gtk_widget_show (GTK_WIDGET (priv->mainwin));
+
+	uapp = unique_app_new ("org.gnome.Brasero", NULL);
+
+	if (unique_app_is_running (uapp))
+	{
+		UniqueResponse response;
+
+		response = unique_app_send_message (uapp, UNIQUE_ACTIVATE, NULL);
+    		g_object_unref (uapp);
+		uapp = NULL;
+
+		/* FIXME: we should tell the user why it did not work. Or is it
+		 * handled by libunique? */
+		return (response == UNIQUE_RESPONSE_OK);
+	}
+
 	gtk_main ();
+
+	return TRUE;
 }
 
 static void
