@@ -30,36 +30,14 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
-#include <gtk/gtklabel.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtksizegroup.h>
-
+#include <gtk/gtk.h>
 
 #include "brasero-file-filtered.h"
 #include "brasero-filter-option.h"
 #include "brasero-utils.h"
-#include "brasero-data-vfs.h"
 
-enum  {
-	STOCK_ID_COL,
-	UNESCAPED_URI_COL,
-	TYPE_COL,
-	STATUS_COL,
-	ACTIVABLE_COL,
-	NB_COL,
-};
+#include "brasero-track-data-cfg.h"
+
 
 typedef struct _BraseroFileFilteredPrivate BraseroFileFilteredPrivate;
 struct _BraseroFileFilteredPrivate
@@ -68,38 +46,31 @@ struct _BraseroFileFilteredPrivate
 	GtkWidget *restore;
 	GtkWidget *options;
 
-	GSList *broken;
-	GSList *hidden;
-	GSList *recursive;
-	GSList *unreadable;
-
-	guint idle_id;
-
-	guint num;
+	BraseroTrackDataCfg *track;
 };
 
 #define BRASERO_FILE_FILTERED_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_FILE_FILTERED, BraseroFileFilteredPrivate))
 
-enum
-{
-	FILTERED_SIGNAL,
-	RESTORED_SIGNAL,
-
-	LAST_SIGNAL
-};
-
-
-static guint file_filtered_signals[LAST_SIGNAL] = { 0 };
-
 G_DEFINE_TYPE (BraseroFileFiltered, brasero_file_filtered, GTK_TYPE_EXPANDER);
 
+enum {
+	PROP_0,
+	PROP_TRACK
+};
 
 static gchar *
-brasero_file_filtered_get_label_text (guint num, gboolean expanded)
+brasero_file_filtered_get_label_text (BraseroFileFiltered *self)
 {
+	guint num;
 	gchar *label;
+	GtkTreeModel *model;
+	BraseroFileFilteredPrivate *priv;
 
-	if (expanded) {
+	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
+	num = gtk_tree_model_iter_n_children (model, NULL);
+
+	if (gtk_expander_get_expanded (GTK_EXPANDER (self))) {
 		if (!num)
 			label = g_strdup (_("No file filtered"));
 		else
@@ -124,10 +95,10 @@ brasero_file_filtered_update (BraseroFileFiltered *self)
 
 	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
 
-	markup = brasero_file_filtered_get_label_text (priv->num,
-						       gtk_expander_get_expanded (GTK_EXPANDER (self)));
+	markup = brasero_file_filtered_get_label_text (self);
+
 	widget = gtk_expander_get_label_widget (GTK_EXPANDER (self));
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), markup);
+ 	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), markup);
 	g_free (markup);
 }
 
@@ -139,200 +110,20 @@ brasero_file_filtered_activate (GtkExpander *self)
 }
 
 void
-brasero_file_filtered_remove (BraseroFileFiltered *self,
-			      const gchar *uri)
+brasero_file_filtered_row_inserted (GtkTreeModel *model,
+				    GtkTreePath *treepath,
+				    GtkTreeIter *iter,
+				    BraseroFileFiltered *self)
 {
-	BraseroFileFilteredPrivate *priv;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	guint len;
-
-	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
-	if (!gtk_tree_model_get_iter_first (model, &iter))
-		return;
-
-	len = strlen (uri);
-	while (1) {
-		gchar *iter_uri;
-
-		iter_uri = NULL;
-		gtk_tree_model_get (model, &iter,
-				    UNESCAPED_URI_COL, &iter_uri,
-				    -1);
-
-		if (!iter_uri)
-			continue;
-
-		if (!strcmp (uri, iter_uri)) {
-			g_free (iter_uri);
-			priv->num --;
-			if (!gtk_list_store_remove (GTK_LIST_STORE (model), &iter))
-				break;
-
-			continue;
-		}
-
-		if (!strncmp (uri, iter_uri, len)
-		&&   iter_uri [len] == G_DIR_SEPARATOR) {
-			g_free (iter_uri);
-			priv->num --;
-			if (!gtk_list_store_remove (GTK_LIST_STORE (model), &iter))
-				break;
-
-			continue;
-		}
-
-		g_free (iter_uri);
-		if (!gtk_tree_model_iter_next (model, &iter))
-			break;
-	}
-
 	brasero_file_filtered_update (self);
 }
 
-static void
-brasero_file_filtered_add_real (BraseroFileFiltered *self,
-				const gchar *unescaped_uri,
-				BraseroFilterStatus status)
-{
-	gchar *labels [] = { N_("hidden file"),
-			     N_("unreadable file"),
-			     N_("broken symlink"),
-			     N_("recursive symlink"),
-			     NULL };
-	BraseroFileFilteredPrivate *priv;
-	const gchar *stock_id;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	gchar *type;
-
-	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
-
-	type = labels [ status - 1 ];
-	if (status == BRASERO_FILTER_UNREADABLE)
-		stock_id = GTK_STOCK_CANCEL;
-	else
-		stock_id = NULL;
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
-	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-
-	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			    STOCK_ID_COL, stock_id,
-			    UNESCAPED_URI_COL, unescaped_uri,
-			    TYPE_COL, _(type),
-			    STATUS_COL, status,
-			    ACTIVABLE_COL, (status != BRASERO_FILTER_UNREADABLE && status != BRASERO_FILTER_RECURSIVE_SYM),
-			    -1);
-
-	priv->num ++;
-}
-
-static gboolean
-brasero_file_filtered_add_loop (gpointer data)
-{
-	GSList *iter;
-	BraseroFileFilteredPrivate *priv;
-	BraseroFileFiltered *self = BRASERO_FILE_FILTERED (data);
-
-	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
-
-	for (iter = priv->hidden; iter; iter = iter->next) {
-		gchar *unescaped_uri;
-
-		unescaped_uri = iter->data;
-		brasero_file_filtered_add_real (self,
-						unescaped_uri,
-						BRASERO_FILTER_HIDDEN);
-		g_free (unescaped_uri);
-	}
-	g_slist_free (priv->hidden);
-	priv->hidden = NULL;
-
-	for (iter = priv->broken; iter; iter = iter->next) {
-		gchar *unescaped_uri;
-
-		unescaped_uri = iter->data;
-		brasero_file_filtered_add_real (self,
-						unescaped_uri,
-						BRASERO_FILTER_BROKEN_SYM);
-		g_free (unescaped_uri);
-	}
-	g_slist_free (priv->broken);
-	priv->broken = NULL;
-
-	for (iter = priv->recursive; iter; iter = iter->next) {
-		gchar *unescaped_uri;
-
-		unescaped_uri = iter->data;
-		brasero_file_filtered_add_real (self,
-						unescaped_uri,
-						BRASERO_FILTER_RECURSIVE_SYM);
-		g_free (unescaped_uri);
-	}
-	g_slist_free (priv->recursive);
-	priv->recursive = NULL;
-
-	for (iter = priv->unreadable; iter; iter = iter->next) {
-		gchar *unescaped_uri;
-
-		unescaped_uri = iter->data;
-		brasero_file_filtered_add_real (self,
-						unescaped_uri,
-						BRASERO_FILTER_UNREADABLE);
-		g_free (unescaped_uri);
-	}
-	g_slist_free (priv->unreadable);
-	priv->unreadable = NULL;
-
-	/* update label */
-	brasero_file_filtered_update (self);
-
-	priv->idle_id = 0;
-	return FALSE;
-}
-
 void
-brasero_file_filtered_add (BraseroFileFiltered *self,
-			   const gchar *uri,
-			   BraseroFilterStatus status)
+brasero_file_filtered_row_deleted (GtkTreeModel *model,
+				   GtkTreePath *treepath,
+				   BraseroFileFiltered *self)
 {
-	BraseroFileFilteredPrivate *priv;
-
-	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
-
-	/* The idea here is to delay the introduction of each file in the tree
-	 * and the label update so as not slow down brasero too much */
-	switch (status) {
-	case BRASERO_FILTER_HIDDEN:
-		priv->hidden = g_slist_prepend (priv->hidden,
-						g_uri_unescape_string (uri, NULL));
-		break;
-
-	case BRASERO_FILTER_BROKEN_SYM:
-		priv->broken = g_slist_prepend (priv->broken,
-						g_uri_unescape_string (uri, NULL));
-		break;
-	case BRASERO_FILTER_RECURSIVE_SYM:
-		priv->recursive = g_slist_prepend (priv->recursive,
-						   g_uri_unescape_string (uri, NULL));
-		break;
-	case BRASERO_FILTER_UNREADABLE:
-		priv->unreadable = g_slist_prepend (priv->unreadable,
-						    g_uri_unescape_string (uri, NULL));
-		break;
-	case BRASERO_FILTER_NONE:
-	case BRASERO_FILTER_UNKNOWN:
-		default:
-		break;
-	}
-
-	if (!priv->idle_id)
-		priv->idle_id = g_timeout_add (1000,
-					       brasero_file_filtered_add_loop,
-					       self);
+	brasero_file_filtered_update (self);
 }
 
 static void
@@ -374,39 +165,13 @@ brasero_file_filtered_restore_pressed_cb (GtkButton *button,
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree));
 	selected = gtk_tree_selection_get_selected_rows (selection, &model);
 
-	/* reverse the list so as to be able to delete properly item */
 	selected = g_list_reverse (selected);
 
 	for (iter = selected; iter; iter = iter->next) {
 		GtkTreePath *treepath;
-		GtkTreeIter treeiter;
-		gchar *escaped_uri;
-		gchar *uri;
 
 		treepath = iter->data;
-		if (!gtk_tree_model_get_iter (model, &treeiter, treepath)) {
-			gtk_tree_path_free (treepath);
-			continue;
-		}
-
-		gtk_tree_path_free (treepath);
-
-		uri = NULL;
-		gtk_tree_model_get (model, &treeiter,
-				    UNESCAPED_URI_COL, &uri, 
-				    -1);
-
-		escaped_uri = g_uri_escape_string (uri,
-						   G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
-						   FALSE);
-		g_signal_emit (self,
-			       file_filtered_signals [RESTORED_SIGNAL],
-			       0,
-			       escaped_uri);
-		g_free (escaped_uri);
-
-		gtk_list_store_remove (GTK_LIST_STORE (model), &treeiter);
-		priv->num --;
+		brasero_track_data_cfg_restore (priv->track, treepath);
 	}
 	g_list_free (selected);
 
@@ -428,51 +193,6 @@ brasero_file_filtered_selection_changed_cb (GtkTreeSelection *selection,
 		gtk_widget_set_sensitive (priv->restore, FALSE);
 }
 
-void
-brasero_file_filtered_clear (BraseroFileFiltered *self)
-{
-	BraseroFileFilteredPrivate *priv;
-	GtkTreeModel *model;
-
-	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
-
-	if (priv->idle_id) {
-		g_source_remove (priv->idle_id);
-		priv->idle_id = 0;
-	}
-
-	if (priv->hidden) {
-		g_slist_foreach (priv->hidden, (GFunc) g_free, NULL);
-		g_slist_free (priv->hidden);
-		priv->hidden = NULL;
-	}
-
-	if (priv->broken) {
-		g_slist_foreach (priv->broken, (GFunc) g_free, NULL);
-		g_slist_free (priv->broken);
-		priv->broken = NULL;
-	}
-
-	if (priv->recursive) {
-		g_slist_foreach (priv->recursive, (GFunc) g_free, NULL);
-		g_slist_free (priv->recursive);
-		priv->recursive = NULL;
-	}
-
-	if (priv->unreadable) {
-		g_slist_foreach (priv->unreadable, (GFunc) g_free, NULL);
-		g_slist_free (priv->unreadable);
-		priv->unreadable = NULL;
-	}
-
-	priv->num = 0;
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
-	gtk_list_store_clear (GTK_LIST_STORE (model));
-
-	brasero_file_filtered_update (self);
-}
-
 void
 brasero_file_filtered_set_right_button_group (BraseroFileFiltered *self,
 					      GtkSizeGroup *group)
@@ -492,7 +212,6 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 	GtkWidget *button;
 	GtkWidget *scroll;
 	GtkWidget *mainbox;
-	GtkListStore *model;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 	BraseroFileFilteredPrivate *priv;
@@ -508,20 +227,12 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_widget_show (hbox);
 
-	model = gtk_list_store_new (NB_COL,
-				    G_TYPE_STRING,
-				    G_TYPE_STRING,
-				    G_TYPE_STRING,
-				    G_TYPE_INT,
-				    G_TYPE_BOOLEAN);
-
-	priv->tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+	priv->tree = gtk_tree_view_new ();
 	gtk_widget_show (priv->tree);
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (priv->tree), TRUE);
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree)),
 				     GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (priv->tree), TRUE);
-	g_object_unref (model);
 
 	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree)),
 						       "changed",
@@ -533,15 +244,20 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_add_attribute (column, renderer,
-					    "stock-id", STOCK_ID_COL);
+					    "stock-id", BRASERO_FILTERED_STOCK_ID_COL);
+	gtk_tree_view_column_add_attribute (column, renderer,
+					    "sensitive", BRASERO_FILTERED_FATAL_ERROR_COL);
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_end (column, renderer, TRUE);
 	gtk_tree_view_column_add_attribute (column, renderer,
-					    "text", UNESCAPED_URI_COL);
+					    "text", BRASERO_FILTERED_URI_COL);
+	gtk_tree_view_column_add_attribute (column, renderer,
+					    "sensitive", BRASERO_FILTERED_FATAL_ERROR_COL);
+
 	gtk_tree_view_column_set_title (column, _("Files"));
 	gtk_tree_view_append_column (GTK_TREE_VIEW (priv->tree), column);
-	gtk_tree_view_column_set_sort_column_id (column, UNESCAPED_URI_COL);
+	gtk_tree_view_column_set_sort_column_id (column, BRASERO_FILTERED_URI_COL);
 	gtk_tree_view_column_set_clickable (column, TRUE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
@@ -549,10 +265,11 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Type"), renderer,
-							   "text", TYPE_COL,
+							   "text", BRASERO_FILTERED_STATUS_COL,
+							   "sensitive", BRASERO_FILTERED_FATAL_ERROR_COL,
 							   NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (priv->tree), column);
-	gtk_tree_view_column_set_sort_column_id (column, TYPE_COL);
+	gtk_tree_view_column_set_sort_column_id (column, BRASERO_FILTERED_STATUS_COL);
 	gtk_tree_view_column_set_clickable (column, TRUE);
 
 	scroll = gtk_scrolled_window_new (NULL, NULL);
@@ -603,39 +320,67 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 }
 
 static void
-brasero_file_filtered_finalize (GObject *object)
+brasero_file_filtered_set_property (GObject *object,
+				    guint property_id,
+				    const GValue *value,
+				    GParamSpec *pspec)
+{
+	BraseroFileFilteredPrivate *priv;
+	GtkTreeModel *model;
+
+	priv = BRASERO_FILE_FILTERED_PRIVATE (object);
+
+	switch (property_id) {
+	case PROP_TRACK: /* Readable and only writable at creation time */
+		priv->track = g_object_ref (g_value_get_object (value));
+		model = brasero_track_data_cfg_get_filtered_model (priv->track);
+		gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree), model);
+		g_object_unref (model);
+
+		g_signal_connect (g_value_get_object (value),
+				  "row-deleted",
+				  G_CALLBACK (brasero_file_filtered_row_deleted),
+				  object);
+		g_signal_connect (g_value_get_object (value),
+				  "row-inserted",
+				  G_CALLBACK (brasero_file_filtered_row_inserted),
+				  object);
+		break;
+
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
+static void
+brasero_file_filtered_get_property (GObject *object,
+				    guint property_id,
+				    GValue *value,
+				    GParamSpec *pspec)
 {
 	BraseroFileFilteredPrivate *priv;
 
 	priv = BRASERO_FILE_FILTERED_PRIVATE (object);
 
-	if (priv->idle_id) {
-		g_source_remove (priv->idle_id);
-		priv->idle_id = 0;
-	}
+	switch (property_id) {
+	case PROP_TRACK:
+		g_value_set_object (value, G_OBJECT (priv->track));
+		break;
 
-	if (priv->hidden) {
-		g_slist_foreach (priv->hidden, (GFunc) g_free, NULL);
-		g_slist_free (priv->hidden);
-		priv->hidden = NULL;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 	}
+}
 
-	if (priv->broken) {
-		g_slist_foreach (priv->broken, (GFunc) g_free, NULL);
-		g_slist_free (priv->broken);
-		priv->broken = NULL;
-	}
+static void
+brasero_file_filtered_finalize (GObject *object)
+{
+	BraseroFileFilteredPrivate *priv;
 
-	if (priv->recursive) {
-		g_slist_foreach (priv->recursive, (GFunc) g_free, NULL);
-		g_slist_free (priv->recursive);
-		priv->recursive = NULL;
-	}
-
-	if (priv->unreadable) {
-		g_slist_foreach (priv->unreadable, (GFunc) g_free, NULL);
-		g_slist_free (priv->unreadable);
-		priv->unreadable = NULL;
+	priv = BRASERO_FILE_FILTERED_PRIVATE (object);
+	if (priv->track) {
+		g_object_unref (priv->track);
+		priv->track = NULL;
 	}
 
 	G_OBJECT_CLASS (brasero_file_filtered_parent_class)->finalize (object);
@@ -650,43 +395,32 @@ brasero_file_filtered_class_init (BraseroFileFilteredClass *klass)
 	g_type_class_add_private (klass, sizeof (BraseroFileFilteredPrivate));
 
 	object_class->finalize = brasero_file_filtered_finalize;
+	object_class->set_property = brasero_file_filtered_set_property;
+	object_class->get_property = brasero_file_filtered_get_property;
 
 	expander_class->activate = brasero_file_filtered_activate;
 
-	file_filtered_signals[FILTERED_SIGNAL] =
-		g_signal_new ("filtered",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
-		              G_STRUCT_OFFSET (BraseroFileFilteredClass, filtered),
-		              NULL, NULL,
-		              g_cclosure_marshal_VOID__STRING,
-		              G_TYPE_NONE, 1,
-		              G_TYPE_STRING);
-
-	file_filtered_signals[RESTORED_SIGNAL] =
-		g_signal_new ("restored",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
-		              G_STRUCT_OFFSET (BraseroFileFilteredClass, restored),
-		              NULL, NULL,
-		              g_cclosure_marshal_VOID__STRING,
-		              G_TYPE_NONE, 1,
-		              G_TYPE_STRING);
+	g_object_class_install_property (object_class,
+					 PROP_TRACK,
+					 g_param_spec_object ("track",
+							      "A BraseroTrackDataCfg",
+							      "The BraseroTrackDataCfg used by the internal GtkTreeView",
+							      BRASERO_TYPE_TRACK_DATA_CFG,
+							      G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY));
 }
 
 GtkWidget*
-brasero_file_filtered_new (void)
+brasero_file_filtered_new (BraseroTrackDataCfg *track)
 {
-	gchar *markup;
 	GtkWidget *object;
 
-	markup = brasero_file_filtered_get_label_text (0, FALSE);
 	object = g_object_new (BRASERO_TYPE_FILE_FILTERED,
-			       "label", markup,
+			       "track", track,
+			       "label", "",
 			       "use-markup", TRUE,
 			       "use-underline", TRUE,
 			       NULL);
-	g_free (markup);
 
+	brasero_file_filtered_update (BRASERO_FILE_FILTERED (object));
 	return object;
 }
