@@ -32,6 +32,9 @@
 
 #include "eggtreemultidnd.h"
 
+#include "brasero-tags.h"
+
+#include "brasero-misc.h"
 #include "burn-debug.h"
 #include "brasero-app.h"
 #include "brasero-disc.h"
@@ -43,6 +46,7 @@
 #include "brasero-multi-song-props.h"
 #include "brasero-song-properties.h"
 #include "brasero-session-cfg.h"
+#include "brasero-track-stream.h"
 
 typedef struct _BraseroVideoDiscPrivate BraseroVideoDiscPrivate;
 struct _BraseroVideoDiscPrivate
@@ -1319,31 +1323,6 @@ brasero_video_disc_get_status (BraseroDisc *self,
 	return brasero_video_project_get_status (project, remaining, current_task);
 }
 
-BraseroDiscResult
-brasero_video_disc_set_session_param (BraseroDisc *self,
-				      BraseroBurnSession *session)
-{
-	BraseroVideoDiscPrivate *priv;
-	BraseroTrackType type;
-	GtkTreeModel *model;
-	GValue *value;
-
-	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
-
-	type.type = BRASERO_TRACK_TYPE_AUDIO;
-	type.subtype.audio_format = BRASERO_AUDIO_FORMAT_UNDEFINED|BRASERO_VIDEO_FORMAT_UNDEFINED;
-	brasero_burn_session_set_input_type (session, &type);
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
-	value = g_new0 (GValue, 1);
-	g_value_init (value, G_TYPE_INT64);
-	g_value_set_int64 (value, brasero_video_project_get_size (BRASERO_VIDEO_PROJECT (model)));
-	brasero_burn_session_tag_add (session,
-				      BRASERO_AUDIO_TRACK_SIZE_TAG,
-				      value);
-	return BRASERO_BURN_OK;
-}
-
 BraseroDiscResult
 brasero_video_disc_set_session_contents (BraseroDisc *self,
 					 BraseroBurnSession *session)
@@ -1367,10 +1346,11 @@ brasero_video_disc_set_session_contents (BraseroDisc *self,
 
 		/* It's good practice to unref the track afterwards as we don't
 		 * need it anymore. BraseroBurnSession refs it. */
-		brasero_track_unref (track);
+		g_object_unref (track);
 
 	}
 	g_slist_free (tracks);
+
 	return BRASERO_DISC_OK;
 }
 
@@ -1383,7 +1363,7 @@ brasero_video_disc_get_track (BraseroDisc *disc,
 	BraseroVideoProject *project;
 	BraseroVideoDiscPrivate *priv;
 
-	disc_track->type = BRASERO_DISC_TRACK_VIDEO;
+	disc_track->type = BRASERO_PROJECT_TYPE_VIDEO;
 
 	priv = BRASERO_VIDEO_DISC_PRIVATE (disc);
 	project = BRASERO_VIDEO_PROJECT (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
@@ -1396,15 +1376,24 @@ brasero_video_disc_get_track (BraseroDisc *disc,
 		track = iter->data;
 
 		song = g_new0 (BraseroDiscSong, 1);
-		song->uri = brasero_track_get_audio_source (track, TRUE);
-		song->start = brasero_track_get_audio_start (track);
-		song->end = brasero_track_get_audio_end (track);
-		song->info = brasero_song_info_copy (brasero_track_get_audio_info (track));
+		song->uri = brasero_track_stream_get_source (BRASERO_TRACK_STREAM (track), TRUE);
+		song->start = brasero_track_stream_get_start (BRASERO_TRACK_STREAM (track));
+		song->end = brasero_track_stream_get_end (BRASERO_TRACK_STREAM (track));
+
+		song->info = g_new0 (BraseroStreamInfo, 1);
+		song->info->artist = g_strdup (brasero_track_tag_lookup_string (BRASERO_TRACK (track),
+										BRASERO_TRACK_STREAM_ARTIST_TAG));
+		song->info->title = g_strdup (brasero_track_tag_lookup_string (BRASERO_TRACK (track),
+									       BRASERO_TRACK_STREAM_TITLE_TAG));
+		song->info->composer = g_strdup (brasero_track_tag_lookup_string (BRASERO_TRACK (track),
+										  BRASERO_TRACK_STREAM_COMPOSER_TAG));
+		song->info->isrc = brasero_track_tag_lookup_int (BRASERO_TRACK (track),
+								 BRASERO_TRACK_STREAM_ISRC_TAG);
 
 		disc_track->contents.tracks = g_slist_append (disc_track->contents.tracks, song);
 	}
 
-	g_slist_foreach (tracks, (GFunc) brasero_track_unref, NULL);
+	g_slist_foreach (tracks, (GFunc) g_object_unref, NULL);
 	g_slist_free (tracks);
 
 	return BRASERO_DISC_OK;
@@ -1418,7 +1407,7 @@ brasero_video_disc_load_track (BraseroDisc *disc,
 	BraseroVideoProject *project;
 	BraseroVideoDiscPrivate *priv;
 
-	g_return_val_if_fail (track->type == BRASERO_DISC_TRACK_VIDEO, FALSE);
+	g_return_val_if_fail (track->type == BRASERO_PROJECT_TYPE_VIDEO, FALSE);
 
 	if (track->contents.tracks == NULL)
 		return BRASERO_DISC_ERROR_EMPTY_SELECTION;
@@ -1453,7 +1442,6 @@ brasero_video_disc_iface_disc_init (BraseroDiscIface *iface)
 	iface->reset = brasero_video_disc_reset;
 
 	iface->get_status = brasero_video_disc_get_status;
-	iface->set_session_param = brasero_video_disc_set_session_param;
 	iface->set_session_contents = brasero_video_disc_set_session_contents;
 
 	iface->get_track = brasero_video_disc_get_track;
