@@ -45,11 +45,13 @@
 
 #include "brasero-project-manager.h"
 #include "brasero-multi-dnd.h"
-#include "brasero-session.h"
 #include "brasero-utils.h"
 #include "brasero-app.h"
+
 #include "burn-debug.h"
-#include "burn.h"
+
+#include "brasero-burn-lib.h"
+#include "brasero-session.h"
 
 #include "eggsmclient.h"
 
@@ -157,12 +159,12 @@ static const GOptionEntry options [] = {
 	function (BRASERO_PROJECT_MANAGER (manager_MACRO), uri);	\
 }
 
-#define BRASERO_PROJECT_OPEN_LIST(manager_MACRO, function, uris)		\
+#define BRASERO_PROJECT_OPEN_LIST(manager_MACRO, function_MACRO, uris_MACROS, burn_MACRO)	\
 {										\
 	GSList *list = NULL;							\
 	gchar **iter;								\
 	/* convert all names into a GSList * */					\
-	for (iter = uris; iter && *iter; iter ++) {				\
+	for (iter = uris_MACROS; iter && *iter; iter ++) {				\
 		gchar *uri;							\
 		GFile *file;							\
 		file = g_file_new_for_commandline_arg (*iter);			\
@@ -172,11 +174,138 @@ static const GOptionEntry options [] = {
 	}									\
 	/* reverse to keep the order of files */				\
 	list = g_slist_reverse (list);						\
-	function (BRASERO_PROJECT_MANAGER (manager_MACRO), list);		\
+	function_MACRO (BRASERO_PROJECT_MANAGER (manager_MACRO), list, burn_MACRO);		\
 	g_slist_foreach (list, (GFunc) g_free, NULL);				\
 	g_slist_free (list);							\
 }
 
+static void
+brasero_handle_burn_uri (BraseroApp *app,
+			 GtkWidget *manager)
+{
+	GFileEnumerator *enumerator;
+	GFileInfo *info = NULL;
+	GError *error = NULL;
+	GSList *list = NULL;
+	GFile *file;
+
+	/* Here we get the contents from the burn:// URI and add them
+	 * individually to the data project. This is done in case it is
+	 * empty no to start the "Getting Project Size" dialog and then
+	 * show the "Project is empty" dialog. Do this synchronously as:
+	 * - we only want the top nodes which reduces time needed
+	 * - it's always local
+	 * - windows haven't been shown yet
+	 * NOTE: don't use any file specified on the command line. */
+	file = g_file_new_for_uri ("burn://");
+	enumerator = g_file_enumerate_children (file,
+						G_FILE_ATTRIBUTE_STANDARD_NAME,
+						G_FILE_QUERY_INFO_NONE,
+						NULL,
+						&error);
+
+	if (!enumerator) {
+		gchar *string;
+
+		if (error)
+			string = g_strdup_printf (_("An internal error occured (%s)"), error->message);
+		else
+			string = g_strdup (_("An internal error occured"));
+
+		brasero_app_alert (app,
+				   _("Error while loading the project."),
+				   string,
+				   GTK_MESSAGE_ERROR);
+
+		g_free (string);
+		g_object_unref (file);
+		return;
+	}
+
+	while ((info = g_file_enumerator_next_file (enumerator, NULL, &error)) != NULL) {
+		list = g_slist_prepend (list, g_strconcat ("burn:///", g_file_info_get_name (info), NULL));
+		g_object_unref (info);
+	}
+
+	g_object_unref (enumerator);
+	g_object_unref (file);
+
+	if (error) {
+		gchar *string;
+
+		if (error)
+			string = g_strdup_printf (_("An internal error occured (%s)"), error->message);
+		else
+			string = g_strdup (_("An internal error occured"));
+
+		brasero_app_alert (app,
+				   _("Error while loading the project."),
+				   string,
+				   GTK_MESSAGE_ERROR);
+
+		g_free (string);
+
+		g_slist_foreach (list, (GFunc) g_free, NULL);
+		g_slist_free (list);
+		return;
+	}
+
+	if (!list) {
+		brasero_app_alert (app,
+				   _("Please add files to the project."),
+				   _("The project is empty"),
+				   GTK_MESSAGE_ERROR);
+		return;
+	}
+
+	/* reverse to keep the order of files */
+	list = g_slist_reverse (list);
+	brasero_app_create_mainwin (app);
+	manager = brasero_app_get_project_manager (app);
+	brasero_project_manager_data (BRASERO_PROJECT_MANAGER (manager), list, TRUE);
+
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_slist_free (list);
+	return;
+}
+
+static gboolean
+brasero_app_open_project (BraseroApp *app,
+			  const gchar *path,
+			  gboolean playlist,
+			  gboolean burn)
+{
+	BraseroDiscTrack *track = NULL;
+	BraseroProjectType type;
+	GtkWidget *manager;
+	GFile *file;
+	gchar *uri;
+
+	brasero_app_create_mainwin (app);
+
+	file = g_file_new_for_commandline_arg (path);
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
+
+#ifdef BUILD_PLAYLIST
+
+	if (playlist)
+		type = brasero_project_open_audio_playlist_project (uri, &track, TRUE);
+	else
+
+#endif
+		
+		type = brasero_project_open_project_xml (uri, &track, TRUE);
+
+	if (type == BRASERO_PROJECT_TYPE_INVALID)
+		return FALSE;
+
+	manager = brasero_app_get_project_manager (app);
+	brasero_project_manager_open_project (BRASERO_PROJECT_MANAGER (manager), track, uri, burn);
+
+	return TRUE;
+}
+
 static void
 brasero_app_parse_options (BraseroApp *app)
 {
@@ -218,7 +347,6 @@ brasero_app_parse_options (BraseroApp *app)
 
 	if (nb > 1) {
 		brasero_app_create_mainwin (app);
-
 		brasero_app_alert (app,
 				   _("Incompatible command line options used."),
 				   _("Only one option can be given at a time"),
@@ -227,6 +355,49 @@ brasero_app_parse_options (BraseroApp *app)
 		manager = brasero_app_get_project_manager (app);
 		brasero_project_manager_empty (BRASERO_PROJECT_MANAGER (manager));
 	}
+	else if (project_uri) {
+		brasero_app_open_project (app, project_uri, FALSE, FALSE);
+	}
+	else if (burn_project_uri) {
+		brasero_app_open_project (app, burn_project_uri, FALSE, TRUE);
+
+		if (g_remove (burn_project_uri) != 0) {
+			gchar *path;
+
+			path = g_filename_from_uri (burn_project_uri, NULL, NULL);
+			g_remove (path);
+			g_free (path);
+		}
+		return;
+	}
+
+#ifdef BUILD_PLAYLIST
+
+	else if (playlist_uri) {
+		brasero_app_open_project (app, playlist_uri, TRUE, FALSE);
+	}
+
+#endif
+
+	else if (open_ncb) {
+		brasero_handle_burn_uri (app, manager);
+		return;
+	}
+	else if (audio_project) {
+		brasero_app_create_mainwin (app);
+		manager = brasero_app_get_project_manager (app);
+		BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_audio, files, FALSE);
+	}
+	else if (data_project) {
+		brasero_app_create_mainwin (app);
+		manager = brasero_app_get_project_manager (app);
+		BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files, FALSE);
+	}
+	else if (video_project) {
+		brasero_app_create_mainwin (app);
+		manager = brasero_app_get_project_manager (app);
+	    	BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_video, files, FALSE);
+	}
 	else if (copy_project) {
 		gchar *device = NULL;
 
@@ -252,138 +423,6 @@ brasero_app_parse_options (BraseroApp *app)
 		brasero_app_burn_image (app, uri);
 		return;
 	}
-	else if (project_uri) {
-		brasero_app_create_mainwin (app);
-		manager = brasero_app_get_project_manager (app);
-		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
-		BRASERO_PROJECT_OPEN_URI (manager, brasero_project_manager_open_project, project_uri);
-	}
-	else if (burn_project_uri) {
-		brasero_app_create_mainwin (app);
-		manager = brasero_app_get_project_manager (app);
-		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
-		BRASERO_PROJECT_OPEN_URI (manager, brasero_project_manager_burn_project, burn_project_uri);
-		if (g_remove (burn_project_uri) != 0) {
-			gchar *path;
-
-			path = g_filename_from_uri (burn_project_uri, NULL, NULL);
-			g_remove (path);
-			g_free (path);
-		}
-		return;
-	}
-	else if (open_ncb) {
-		GFileEnumerator *enumerator;
-		GFileInfo *info = NULL;
-		GError *error = NULL;
-		GSList *list = NULL;
-		GFile *file;
-
-		/* Here we get the contents from the burn:// URI and add them
-		 * individually to the data project. This is done in case it is
-		 * empty no to start the "Getting Project Size" dialog and then
-		 * show the "Project is empty" dialog. Do this synchronously as:
-		 * - we only want the top nodes which reduces time needed
-		 * - it's always local
-		 * - windows haven't been shown yet
-		 * NOTE: don't use any file specified on the command line. */
-		file = g_file_new_for_uri ("burn://");
-		enumerator = g_file_enumerate_children (file,
-							G_FILE_ATTRIBUTE_STANDARD_NAME,
-							G_FILE_QUERY_INFO_NONE,
-							NULL,
-							&error);
-
-		if (!enumerator) {
-			gchar *string;
-
-			if (error)
-				string = g_strdup_printf (_("An internal error occured (%s)"), error->message);
-			else
-				string = g_strdup (_("An internal error occured"));
-
-			brasero_app_alert (app,
-					   _("Error while loading the project."),
-					   string,
-					   GTK_MESSAGE_ERROR);
-
-			g_free (string);
-			g_object_unref (file);
-			return;
-		}
-
-		while ((info = g_file_enumerator_next_file (enumerator, NULL, &error)))
-			list = g_slist_prepend (list, g_strconcat ("burn:///", g_file_info_get_name (info), NULL));
-
-		g_object_unref (enumerator);
-		g_object_unref (file);
-
-		if (error) {
-			gchar *string;
-
-			if (error)
-				string = g_strdup_printf (_("An internal error occured (%s)"), error->message);
-			else
-				string = g_strdup (_("An internal error occured"));
-
-			brasero_app_alert (app,
-					   _("Error while loading the project."),
-					   string,
-					   GTK_MESSAGE_ERROR);
-
-			g_free (string);
-			g_object_unref (file);
-
-			g_slist_foreach (list, (GFunc) g_free, NULL);
-			g_slist_free (list);
-			return;
-		}
-
-		if (!list) {
-			brasero_app_alert (app,
-					   _("Please add files to the project."),
-					   _("The project is empty"),
-					   GTK_MESSAGE_ERROR);
-			return;
-		}
-
-		/* reverse to keep the order of files */
-		list = g_slist_reverse (list);
-		brasero_app_create_mainwin (app);
-		manager = brasero_app_get_project_manager (app);
-		brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
-		brasero_project_manager_data (BRASERO_PROJECT_MANAGER (manager), list);
-
-		g_slist_foreach (list, (GFunc) g_free, NULL);
-		g_slist_free (list);
-		return;
-	}
-
-#ifdef BUILD_PLAYLIST
-
-	else if (playlist_uri) {
-		brasero_app_create_mainwin (app);
-		manager = brasero_app_get_project_manager (app);
-		BRASERO_PROJECT_OPEN_URI (manager, brasero_project_manager_open_playlist, playlist_uri);
-	}
-
-#endif
-
-	else if (audio_project) {
-		brasero_app_create_mainwin (app);
-		manager = brasero_app_get_project_manager (app);
-		BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_audio, files);
-	}
-	else if (data_project) {
-		brasero_app_create_mainwin (app);
-		manager = brasero_app_get_project_manager (app);
-		BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files);
-	}
-	else if (video_project) {
-		brasero_app_create_mainwin (app);
-		manager = brasero_app_get_project_manager (app);
-	    	BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_video, files);
-	}
 	else if (disc_blank) {
 		gchar *device = NULL;
 
@@ -419,17 +458,15 @@ brasero_app_parse_options (BraseroApp *app)
 		if (g_strv_length (files) == 1) {
 			BraseroProjectType type;
 
-			brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
 			type = brasero_project_manager_open_uri (BRASERO_PROJECT_MANAGER (manager), files [0]);
 
 			/* Fallback if it hasn't got a suitable URI */
-			if (type == BRASERO_PROJECT_TYPE_INVALID)
-				BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files);
-		}
-		else {
-			brasero_project_manager_set_oneshot (BRASERO_PROJECT_MANAGER (manager), TRUE);
-			BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files);
+			if (type == BRASERO_PROJECT_TYPE_INVALID) {
+				BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files, FALSE);
+			}
 		}
+		else
+			BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files, FALSE);
 	}
 	else {
 		brasero_app_create_mainwin (app);
@@ -476,6 +513,7 @@ main (int argc, char **argv)
 
 	g_option_context_add_group (context, egg_sm_client_get_option_group ());
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_add_group (context, brasero_media_get_option_group ());
 	g_option_context_add_group (context, gst_init_get_option_group ());
 	if (g_option_context_parse (context, &argc, &argv, NULL) == FALSE) {
 		g_print (_("Please type %s --help to see all available options\n"), argv [0]);
@@ -497,10 +535,9 @@ main (int argc, char **argv)
 			      NULL);
 
 	brasero_burn_set_debug (debug);
-	brasero_burn_library_init ();
+	brasero_burn_library_start ();
 
 	brasero_enable_multi_DND ();
-	brasero_utils_init ();
 
 	current_app = brasero_app_new ();
 	if (current_app == NULL)
@@ -509,17 +546,11 @@ main (int argc, char **argv)
 	brasero_app_parse_options (current_app);
 	current_app = NULL;
 
-	brasero_burn_library_shutdown ();
+	brasero_burn_library_stop ();
 
 	gconf_client_remove_dir (client, BRASERO_CONF_DIR, NULL);
 	g_object_unref (client);
 
-#ifdef BUILD_GNOME2
-
-	g_object_unref (program);
-
-#endif
-
 	gst_deinit ();
 
 	return 0;
